"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Event = void 0;
const internal_1 = require("../internal");
class Event {
    constructor(agileInstance, config = {}) {
        this.callbacks = new Set(); // Stores callback functions
        this.uses = 0; // How often the event has been used
        this.queue = []; // Queue if something is currently in timeout
        this.enabled = true;
        this.agileInstance = () => agileInstance;
        // Assign defaults to config
        this.config = internal_1.defineConfig(config, {
            enabled: true
        });
        // Set Key
        this._key = this.config.key;
        // Set Enabled
        if (this.config.enabled !== undefined)
            this.enabled = this.config.enabled;
    }
    set key(value) {
        this._key = value;
    }
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // On
    //=========================================================================================================
    /**
     * Register Callback Function
     */
    on(callback) {
        const cleanUpFunction = () => this.unsub(callback);
        // Add callback to Event Callbacks
        this.callbacks.add(callback);
        return cleanUpFunction;
    }
    //=========================================================================================================
    // Trigger
    //=========================================================================================================
    /**
     * Run all callback Functions
     */
    trigger(payload) {
        // If event is disabled, return
        if (!this.enabled)
            return this;
        if (this.config.delay)
            this.delayedTrigger(payload);
        else
            this.normalTrigger(payload);
        return this;
    }
    //=========================================================================================================
    // Disable
    //=========================================================================================================
    /**
     * Disables the Event
     */
    disable() {
        this.enabled = false;
        return this;
    }
    //=========================================================================================================
    // Enable
    //=========================================================================================================
    /**
     * Enables the Event
     */
    enable() {
        this.enabled = true;
        return this;
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * Resets the Event
     */
    reset() {
        // Set Enabled
        this.enabled = this.config.enabled || true;
        // Reset Uses
        this.uses = 0;
        // Clear active timeout
        clearTimeout(this.currentTimeout);
        return this;
    }
    //=========================================================================================================
    // Unsub
    //=========================================================================================================
    /**
     * @internal
     * Unsubs a callback
     */
    unsub(callback) {
        this.callbacks.delete(callback);
    }
    //=========================================================================================================
    // Normal Trigger
    //=========================================================================================================
    /**
     * @internal
     * Call event instantly
     */
    normalTrigger(payload) {
        // Call callbacks
        this.callbacks.forEach(callback => callback(payload));
        // Increase uses
        this.uses++;
        // Check if maxUses has been reached, if so disable event
        if (this.config.maxUses && this.uses >= this.config.maxUses)
            // Disable Event
            this.disable();
    }
    //=========================================================================================================
    // Delayed Trigger
    //=========================================================================================================
    /**
     * @internal
     * Call event after config.delay
     */
    delayedTrigger(payload) {
        // Check if a timeout is currently active if so add the payload to a queue
        if (this.currentTimeout !== undefined) {
            if (payload)
                this.queue.push(payload);
            return;
        }
        // Looper function which loops through the queue(payloads)
        const looper = (payload) => {
            this.currentTimeout = setTimeout(() => {
                // Reset currentTimeout
                this.currentTimeout = undefined;
                // Call normalTrigger
                this.normalTrigger(payload);
                // If items are in queue, continue with them
                if (this.queue.length > 0)
                    looper(this.queue.shift());
            }, this.config.delay);
        };
        // Call looper with current Payload
        looper(payload);
        return;
    }
}
exports.Event = Event;
