"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubController = void 0;
const ComponentSubscriptionContainer_1 = require("./ComponentSubscriptionContainer");
const CallbackSubscriptionContainer_1 = require("./CallbackSubscriptionContainer");
//=========================================================================================================
// Controller
//=========================================================================================================
class SubController {
    constructor(agileInstance) {
        // Component based Subscription
        this.components = new Set();
        // Callback based Subscription
        this.callbacks = new Set();
        this.agileInstance = () => agileInstance;
    }
    //=========================================================================================================
    // Subscribe with Subs Object
    //=========================================================================================================
    /**
     * Subscribe to Agile State with a returned object of props this props can than be returned by the component (See react-integration)
     */
    subscribeWithSubsObject(subscriptionInstance, subs = {}) {
        const subscriptionContainer = this.registerSubscription(subscriptionInstance);
        const props = {};
        subscriptionContainer.passProps = true;
        subscriptionContainer.propStates = Object.assign({}, subs);
        // Go through subs
        let localKeys = Object.keys(subs);
        localKeys.forEach(key => {
            const state = subs[key];
            // Add State to SubscriptionContainer Subs
            subscriptionContainer.subs.add(state);
            // Add SubscriptionContainer to State Subs
            state.dep.subscribe(subscriptionContainer);
            // Add state to props
            props[key] = state.value;
        });
        return {
            subscriptionContainer: subscriptionContainer,
            props: props
        };
    }
    //=========================================================================================================
    // Subscribe with Subs Array
    //=========================================================================================================
    /**
     * Subscribe to Agile State
     */
    subscribeWithSubsArray(subscriptionInstance, subs = []) {
        const subscriptionContainer = this.registerSubscription(subscriptionInstance, subs);
        subs.forEach(state => {
            // Add State to SubscriptionContainer Subs
            subscriptionContainer.subs.add(state);
            // Add SubscriptionContainer to State Dependencies Subs
            state.dep.subscribe(subscriptionContainer);
        });
        return subscriptionContainer;
    }
    //=========================================================================================================
    // Unsubscribe
    //=========================================================================================================
    /**
     * Unsubscribe a component or callback
     */
    unsubscribe(subscriptionInstance) {
        const unsub = (subscriptionContainer) => {
            subscriptionContainer.ready = false;
            // Removes SubscriptionContainer from State subs
            subscriptionContainer.subs.forEach(state => {
                state.dep.unsubscribe(subscriptionContainer);
            });
        };
        // Check if subscriptionInstance is CallbackSubscriptionContainer
        if (subscriptionInstance instanceof CallbackSubscriptionContainer_1.CallbackSubscriptionContainer)
            unsub(subscriptionInstance);
        // Check if subscriptionInstance has componentSubscriptionContainer.. which holds an instance of a ComponentSubscriptionContainer
        if (subscriptionInstance.componentSubscriptionContainer)
            unsub(subscriptionInstance.componentSubscriptionContainer);
    }
    //=========================================================================================================
    // Register Subscription
    //=========================================================================================================
    /**
     * Registers the Component/Callback Subscription and returns a SubscriptionContainer
     */
    registerSubscription(integrationInstance, subs = []) {
        if (typeof integrationInstance === 'function')
            return this.registerCallbackSubscription(integrationInstance, subs);
        return this.registerComponentSubscription(integrationInstance, subs);
    }
    //=========================================================================================================
    // Mount
    //=========================================================================================================
    /**
     * This will mount the component (Mounts currently only useful in Component based Subscription)
     */
    mount(integrationInstance) {
        if (!integrationInstance.componentContainer)
            return;
        // Set Ready to true
        integrationInstance.componentContainer.ready = true;
    }
    //=========================================================================================================
    // Register Component Subscription
    //=========================================================================================================
    /**
     * Registers Component Subscription
     * Note: Helper Function
     */
    registerComponentSubscription(componentInstance, subs = []) {
        // Create ComponentSubscriptionContainer
        const componentContainer = new ComponentSubscriptionContainer_1.ComponentSubscriptionContainer(componentInstance, new Set(subs));
        // Create an instance of the componentSubscriptionContainer in the Component.. to be able to unsubscribe it later (see unsubscribe)
        if (componentInstance.componentSubscriptionContainer)
            componentInstance.componentSubscriptionContainer = componentContainer;
        // Add to components
        this.components.add(componentContainer);
        // Set Ready
        if (!this.agileInstance().config.waitForMount)
            componentContainer.ready = true;
        if (this.agileInstance().config.logJobs)
            console.log("Agile: Registered Component ", componentContainer);
        return componentContainer;
    }
    //=========================================================================================================
    // Register Callback Subscription
    //=========================================================================================================
    /**
     * Registers Callback Subscription
     * Note: Helper Function
     */
    registerCallbackSubscription(callbackFunction, subs = []) {
        // Create CallbackSubscriptionContainer
        const callbackContainer = new CallbackSubscriptionContainer_1.CallbackSubscriptionContainer(callbackFunction, new Set(subs));
        // Add to callbacks
        this.callbacks.add(callbackContainer);
        // Set Ready
        callbackContainer.ready = true;
        if (this.agileInstance().config.logJobs)
            console.log("Agile: Registered Callback ", callbackContainer);
        return callbackContainer;
    }
}
exports.SubController = SubController;
